using System;
using Dakota.Data;
using Dakota.Renderables;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Nez;
using Nez.Particles;

namespace Dakota.Space
{
    public enum ShipUpgrades
    {
        InertialThrusters, // reverse-thrust to create false feeling of inertia
        HyperspaceStabilizer // reduce likelihood of hyperspace malfunction
    }

    public enum ShipWeapons
    {
        PhaseBlaster, // simple blaster
        LaserBeam, // steady pulsed beam
        RailGun // unstable spray 
    }

    // one idea is to have these enter R&D and you can get an imperfect version that gets better as you use it
    // if you're gonna have all these sweet upgrades, better have fun shit to fly near

    public class PlayerShip : Component, IUpdatable, ITriggerListener
    {
        private VirtualButton _fireInput;
        private VirtualButton _secondaryFireInput;
        private VirtualIntegerAxis _rotateInput;
        private VirtualButton _thrustInput;
        private VirtualButton _warpInput;
        private float _emitCounter;
        private ParticleEmitter _emitter;
        private ParticleEmitterConfig _emitterConfig;
        private VectorSprite _booster;
        private float _autothrustTime;

        void ITriggerListener.OnTriggerEnter(Collider other, Collider self)
        {
            if (other.Entity.Name == "asteroid")
            {
                var scene = (SpaceScene) Entity.Scene;
                scene.PlayerHit();
                scene.HitAsteroid(other.Entity, self.Entity.Position);
            }
        }

        void ITriggerListener.OnTriggerExit(Collider other, Collider self)
        {
        }

        private bool ThrustOn => _thrustInput.IsDown || _autothrustTime > 0;

        public void Thrust(float time)
        {
            _autothrustTime = time;
        }

        public void Update()
        {
            var obj = Entity.GetComponent<SpaceObject>();
            if (_fireInput.IsPressed) ((SpaceScene) Entity.Scene).FirePrimary();
            if (_warpInput.IsPressed) ((SpaceScene) Entity.Scene).Warp();

            obj.SetAcceleration(ThrustOn ? GameConstants.ShipAcceleration : 0);
            _booster.Enabled = ThrustOn;

            if (ThrustOn)
                _emitCounter += Time.DeltaTime;
            while (_emitCounter > 0.01f)
            {
                _emitter.Emit(1);
                _emitCounter -= 0.01f;
            }

            obj.TurnVelocity = _rotateInput.Value * GameConstants.ShipTurnSpeed;

            _emitterConfig.Angle = MathHelper.ToDegrees(Entity.Rotation) + 90;
            _emitter.LocalOffset = new Vector2(-16 * (float) Math.Sin(Entity.Rotation),
                16 * (float) Math.Cos(Entity.Rotation));
        }

        public override void OnAddedToEntity()
        {
            Entity.AddComponent(new SpaceObject());

            var shipPolygon = new[]
            {
                new Vector2(0, -16),
                new Vector2(-16, 16),
                new Vector2(0, 10),
                new Vector2(16, 16)
            };
            Entity.AddComponent(new VectorSprite(shipPolygon, GameConstants.ShipColor));

            var boostPoly = new[]
            {
                new Vector2(-16, 16),
                new Vector2(-8, 24),
                new Vector2(0, 12),
                new Vector2(8, 24),
                new Vector2(16, 16)
            };
            _booster = Entity.AddComponent(new VectorSprite(boostPoly, GameConstants.BoostColor));

            SetupInput();

            // don't need indent for collider, also need own copy
            // since PolygonCollider modifies the array for rotations
            var colliderPolygon = new[]
            {
                new Vector2(0, -16),
                new Vector2(-16, 16),
                new Vector2(16, 16)
            };
            Entity.AddComponent(new PolygonCollider(colliderPolygon)).IsTrigger = true;

            var ec = new ParticleEmitterConfig();
            ec.SourcePositionVariance = new Vector2(2, 2);
            ec.Speed = 30;
            ec.SpeedVariance = 1;
            ec.ParticleLifespan = 3;
            ec.ParticleLifespanVariance = 1;
            ec.Angle = 180;
            ec.AngleVariance = 0;
            ec.StartColor = Color.Orange;
            ec.FinishColor = Color.Transparent;
            ec.StartParticleSize = 4;
            ec.FinishParticleSize = 0;
            ec.MaxParticles = 200;
            ec.EmitterType = ParticleEmitterType.Gravity;
            _emitterConfig = ec;
            _emitter = Entity.AddComponent(new ParticleEmitter(ec));
            _emitter.LocalOffset = new Vector2(0, 16);
        }

        private void SetupInput()
        {
            _fireInput = new VirtualButton();
            _fireInput.Nodes.Add(new VirtualButton.KeyboardKey(Keys.Space));
            _fireInput.SetRepeat(0.5f, 0.5f);

            _secondaryFireInput = new VirtualButton();
            _secondaryFireInput.Nodes.Add(new VirtualButton.KeyboardKey(Keys.LeftShift));

            _thrustInput = new VirtualButton();
            _thrustInput.Nodes.Add(new VirtualButton.KeyboardKey(Keys.Up));

            _warpInput = new VirtualButton();
            _warpInput.Nodes.Add(new VirtualButton.KeyboardKey(Keys.X));

            _rotateInput = new VirtualIntegerAxis();
            _rotateInput.Nodes.Add(new VirtualAxis.KeyboardKeys(VirtualInput.OverlapBehavior.TakeNewer, Keys.Left,
                Keys.Right));
        }
    }
}