using System;
using Dakota.Data;
using Dakota.Missions;
using Dakota.Renderables;
using Dakota.Utils;
using Microsoft.Xna.Framework;
using Nez;
using Nez.Particles;
using Random = Nez.Random;

namespace Dakota.Space
{
    internal enum GameMode
    {
        Active,
        Paused
    }

    public class SpaceScene : Scene
    {
        private const int HudWidth = 280;

        private const int PauseFreezeTag = 100;

        // entities
        private Entity _galacticMap;
        public Entity Player;
        public DialogBox DialogBox;
        public TimedVectorText CenterText { get; }
        public BaddieEmitter AsteroidEmitter;

        // game data
        private GameMode _mode;
        public SpaceMission CurrentMission;
        public Zone CurrentZone;
        public PrimaryWeapon Primary;

        public SpaceScene()
        {
            // set up screen & camera
            SetDesignResolution(1280, 720, SceneResolutionPolicy.ShowAllPixelPerfect);
            Screen.SetSize(1280, 720);
            ClearColor = GameConstants.ColorBlack;
            // render background, then everything else, then UI layers
            AddRenderer(new ScreenSpaceRenderer(0, GameConstants.BackgroundRenderLayer));
            AddRenderer(new RenderLayerExcludeRenderer(10, GameConstants.UiRenderLayerBg,
                GameConstants.BackgroundRenderLayer, GameConstants.UiRenderLayerFg));
            AddRenderer(new ScreenSpaceRenderer(100, GameConstants.UiRenderLayerBg, GameConstants.UiRenderLayerFg));
            Core.ExitOnEscapeKeypress = false;
            if (Environment.GetEnvironmentVariable("DEBUG_RENDERING") != null)
                Core.DebugRenderEnabled = true;

            // create player and focus camera
            Player = CreateEntity("player");
            Player.SetTag(PauseFreezeTag);
            Player.AddComponent(new PlayerShip());
            var follow = Camera.Entity.AddComponent(new FollowCamera(Player));
            follow.FocusOffset = new Vector2(-100, 0);

            AddSceneComponent(new SpaceControls());
            AsteroidEmitter = AddSceneComponent(new BaddieEmitter());

            var starField = CreateEntity("StarField").AddComponent(new StarField());
            starField.RenderLayer = GameConstants.BackgroundRenderLayer;

            _galacticMap = CreateEntity("GalacticMap", new Vector2(10, 10));
            _galacticMap.AddComponent<GalacticMap>();

            // create UI
            var hud = CreateEntity("HUD");
            hud.Position = new Vector2(Screen.Width - HudWidth, 0);
            hud.AddComponent(new SpaceHud());
            DialogBox = CreateEntity("dialog box", new Vector2(1, Screen.Height - 100))
                .AddComponent(new DialogBox());
            CenterText = CreateEntity("center text", new Vector2(300, 300))
                .AddComponent(new TimedVectorText(20, GameConstants.SpaceHudFontColor));

            CurrentMission = new Mission1();
            AddSceneComponent(CurrentMission);
            CurrentMission.OnStart();

            _mode = GameMode.Paused;
            TogglePause();
        }

        public void TogglePause()
        {
            if (_mode == GameMode.Active)
            {
                _mode = GameMode.Paused;
                _galacticMap.Enabled = true;

                foreach (var e in FindEntitiesWithTag(PauseFreezeTag))
                    e.Enabled = false;
            }
            else
            {
                _mode = GameMode.Active;
                _galacticMap.Enabled = false;

                foreach (var e in FindEntitiesWithTag(PauseFreezeTag))
                    e.Enabled = true;
            }
        }

        public void PlayerHit()
        {
            CurrentMission.OnPlayerHit();
        }

        public void PlayerWarpTo(Vector2 pos)
        {
            Player.Position = pos;
            Camera.Position = Player.Position - Camera.Entity.GetComponent<FollowCamera>().FocusOffset;
        }

        public Vector2 ZoneShift(int xShift, int yShift)
        {
            if (CurrentMission.ZoneShiftDisabled)
                return Player.Position;

            var newZone = ZoneData.GetZone(
                CurrentZone.UniversalX + xShift,
                CurrentZone.UniversalY + yShift
            );

            var newPos = Player.Position;

            // wrap around or scale to new area (so middle = middle)
            newPos.X = xShift switch
            {
                -1 => newZone.Width,
                1 => 0,
                _ => newPos.X / CurrentZone.Width * newZone.Width
            };
            newPos.Y = yShift switch
            {
                -1 => newZone.Height,
                1 => 0,
                _ => newPos.Y / CurrentZone.Height * newZone.Height
            };

            CurrentZone = newZone;
            return newPos;
        }

        public Entity AddAsteroid(Vector2 position, AsteroidSize size, Vector2? velocity = null)
        {
            var asteroid = CreateEntity("asteroid", position);
            asteroid.SetTag(PauseFreezeTag);
            asteroid.AddComponent(new Asteroid(size));
            var so = asteroid.AddComponent(new SpaceObject());
            so.TurnVelocity = Random.MinusOneToOne() * 3;
            so.Velocity = velocity ?? Random.NextUnitVector() * 20;
            return asteroid;
        }

        public void HitAsteroid(Entity asteroid, Vector2 hitLocation)
        {
            var cur = asteroid.GetComponent<Asteroid>();
            var angle = DakotaUtils.ToAngle(asteroid.Position - hitLocation) + (float) Math.PI;
            // ~ 135 degrees
            var angle1 = DakotaUtils.VecFromAngle(angle - 2.4f) * 50;
            var angle2 = DakotaUtils.VecFromAngle(angle + 2.4f) * 50;

            switch (cur.Size)
            {
                case AsteroidSize.HUGE:
                    AddAsteroid(asteroid.Position, AsteroidSize.LARGE, angle1);
                    AddAsteroid(asteroid.Position + angle2, AsteroidSize.LARGE, angle2);
                    break;
                case AsteroidSize.LARGE:
                    AddAsteroid(asteroid.Position, AsteroidSize.MEDIUM, angle1);
                    AddAsteroid(asteroid.Position + angle2, AsteroidSize.MEDIUM, angle2);
                    break;
                case AsteroidSize.MEDIUM:
                    AddAsteroid(asteroid.Position, AsteroidSize.SMALL, angle1);
                    AddAsteroid(asteroid.Position + angle2, AsteroidSize.SMALL, angle2);
                    break;
                case AsteroidSize.SMALL:
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            var ec = new ParticleEmitterConfig();
            ec.SourcePositionVariance = new Vector2(cur.Radius(), cur.Radius());
            ec.Speed = 30;
            ec.SpeedVariance = 1;
            ec.ParticleLifespan = 10;
            ec.ParticleLifespanVariance = 1;
            ec.Angle = 90;
            ec.AngleVariance = 360;
            ec.StartColor = GameConstants.AsteroidColor;
            ec.FinishColor = Color.Transparent;
            ec.StartParticleSize = 4;
            ec.FinishParticleSize = 0;
            ec.MaxParticles = 200;
            ec.EmitterType = ParticleEmitterType.Gravity;
            var ee = CreateEntity("dust emitter");
            ee.Position = asteroid.Position;
            ee.AddComponent(new ParticleEmitter(ec)).Emit(100);
            // TODO: ensure this gets removed?

            asteroid.Destroy();
            CurrentMission.OnAsteroidDestroyed(cur.Size);
        }

        public void FirePrimary()
        {
            var playerHeading = DakotaUtils.VecFromAngle(Player.Rotation);
            var bullet = CreateEntity("bullet", Player.Position + playerHeading * 20);
            bullet.SetTag(PauseFreezeTag);
            bullet.AddComponent(new Projectile(Primary.Projectile, true, Primary.Duration));
            var so = bullet.AddComponent(new SpaceObject());
            so.Velocity = playerHeading * Primary.Speed;
            if (Primary.Projectile != ProjectileType.Beam)
                so.Velocity += Player.GetComponent<SpaceObject>().Velocity;
        }

        public void Warp()
        {
            // TODO
        }
    }
}