using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Nez;

namespace Lighthouse
{
    public struct Star
    {
        public double R;
        public double Theta;
        public Color Color;

        public Star(double r, double theta, Color color)
        {
            R = r;
            Theta = theta;
            Color = color;
        }
    }

    public class SkyRenderer : RenderableComponent, IUpdatable
    {
        public const int CycleDays = 20;
        
        private Texture2D _moon;
        private Texture2D _moonShadow;
        private Texture2D _dot;
        private Star[] _stars;
        private int _northPoleX;
        private int _northPoleY;
        private int _moonDay;
        private World _world;

        public override float Width => 100;
        public override float Height => 100;

        public SkyRenderer(World world)
        {
            _world = world;
            
            _northPoleX = 300;
            _northPoleY = 300;
            // force first update
            _moonDay = -1;

            _stars = new[]
            {
                new Star(5, 0.1, new Color(205, 205, 205)),
                new Star(10, 0.4, new Color(205, 205, 205)),
                new Star(20, 1, new Color(205, 205, 205)),
                new Star(30, 1.3, new Color(205, 205, 205)),
                new Star(40, 2.0, new Color(205, 205, 205)),
                new Star(40, 2.4, new Color(205, 205, 205)),
                new Star(50, 2.9, new Color(205, 205, 205)),
                new Star(70, 4, new Color(205, 205, 205)),
                new Star(80, 5, new Color(205, 205, 205)),
                new Star(90, 6, new Color(205, 205, 205)),
            };
        }

        public override void OnAddedToEntity()
        {
            _moon = Entity.Scene.Content.LoadTexture("Textures/Moon");
            _moonShadow = new Texture2D(Core.GraphicsDevice, _moon.Width, _moon.Height, false, _moon.Format);
            _dot = new Texture2D(Core.GraphicsDevice, 1, 1);
            _dot.SetData(new[] { Color.White });
            CalculateMoonShadow();
        }

        private void CalculateMoonShadow()
        {
            /*
                 0: New Moon (100% obscured)
                 Waxing Crescent (over-filled from left)
                 0.25: First Quarter (50% filled from left)
                 Waxing Gibbous (under-filled from left)
                 0.5: Full Moon (100% unobscured)
                 Waning Gibbous (under-filled from right)
                 0.75: Third Quarter (50% filled from right)
                 Waning Crescent (over-filled from right)
            */
            if (_moonDay == _world.Day)
                return;
            _moonDay = _world.Day;
            float amount = ((float)_moonDay % CycleDays / CycleDays);
            var dataColors = new Color[_moon.Width*_moon.Height];
            _moon.GetData(dataColors);
            //Console.WriteLine($"updating moon {_moonDay} {_world.Day} {amount}");

            Color left;
            Color right;
            double percent;
            
            if (amount <= 0.5)
            {
                // wind down from left -> right
                percent = 1 - (amount / 0.5);
                left = new Color(0, 0, 0, 200);
                right = new Color(0, 0, 0, 0);
            }
            else
            {
                // fill from right -> left
                percent = 1 - ((amount-0.5) / 0.5);
                left = new Color(0, 0, 0, 0);
                right = new Color(0, 0, 0, 200);
            }

            int firstPixel;
            int lastPixel;
            for (var y = 0; y < _moon.Height; ++y)
            {
                firstPixel = -1;
                lastPixel = _moon.Width;
                for (var x = 0; x < _moon.Width; ++x)
                {
                    if (dataColors[x + y * _moon.Width].A > 0)
                    {
                        if(firstPixel == -1)
                            firstPixel = x;
                        lastPixel = x;
                    }
                }

                for (var x = firstPixel; x <= lastPixel; ++x)
                {
                    dataColors[x + y * _moon.Width] = (float)(x-firstPixel)/(lastPixel-firstPixel) <= percent ? left : right;
                }
            }
            _moonShadow.SetData(dataColors);
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            var t = _world.DayTime/24*2*Math.PI;
            // stars
            foreach(var star in _stars)
                batcher.Draw(_dot,
                    new Rectangle((int) (_northPoleX + star.R * Math.Sin(star.Theta+t)),
                        (int) (_northPoleY + star.R * Math.Cos(star.Theta+t)), 2, 2), star.Color);
            
            // moon
            batcher.Draw(_moon, new Rectangle(50, 50, 96, 96), Color.White);
            batcher.Draw(_moonShadow, new Rectangle(50, 50, 96, 96), Color.White);
        }

        public void Update()
        {
            CalculateMoonShadow();
        }
    }
}